/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: StopJMXServerThread.java 10:14:10 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.servlet;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.apache.commons.io.FileUtils;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.kernel.server.PetalsServer;

/**
 * Servlet with launch and stop the Petals Platform
 * 
 * @author rnaudin
 */
public class PetalsServlet extends HttpServlet {

    /**
     * Unique ID
     */
    private static final long serialVersionUID = -244680354549388172L;

    /**
     * The Petals Server instance
     */
    private static PetalsServer petalsServer;

    @Override
    public void init() throws ServletException {

        if (checkPetalsHome()) {
            getServletContext().log("Petals is starting...");

            try {
                petalsServer = new PetalsServer();
                petalsServer.start();
            } catch (PetalsException e) {
                throw new ServletException(e);
            }
            getServletContext().log(
                "##### Petals container correctly started #####");
        }
    }

    @Override
    public void destroy() {
        try {
            getServletContext().log("Petals is stopping...");
            petalsServer.triggerStop(false);
            getServletContext().log("Petals is stopped...");
        } catch (Exception e) {
            // The Petals Server may already be stopped from the JMX interface
            if (!e.getMessage().equals("Petals admin can not be accessed.")) {
                System.err.println("Problem while stopping Petals");
                e.printStackTrace();
            }
        }
        
        super.destroy();
    }

    private boolean checkPetalsHome() {
        String petalsHomePath = System.getProperty("petals.home");
        if ( petalsHomePath == null ) {
            petalsHomePath = System.getenv("PETALS_HOME");
        }
        
        if ( petalsHomePath == null ) {
            System.setProperty("petals.home", getServletContext().getRealPath(
                "/")
                + "WEB-INF" + File.separator + "petals-root");
        } else {
            // create the targeted petals.home structure if it doen't exists
            File targetPetalsHome = new File(petalsHomePath);
            File sourcePetalsHome = new File( getServletContext().getRealPath(
            "/")
            + "WEB-INF" + File.separator + "petals-root");
            
            
            if ( targetPetalsHome.exists() ) {
                if ( targetPetalsHome.listFiles().length > 0 ) {
                    return true;
                }
            } else {
                if ( !targetPetalsHome.mkdir() ) {
                    getServletContext().log("Failed to create the petals.home directory");
                    return false;
                }
            }
            
            try {
                FileUtils.copyDirectory(sourcePetalsHome, targetPetalsHome);
            } catch (IOException e) {
                getServletContext().log("Failed to create the petals.home files structure : " + e.getMessage());
                return false;
            }
        }
        
        return true;
    }
    
}
